{% macro transform_multipart(property, source, destination) %}
{% if not property.required %}
{{ destination }} = {{source}} if isinstance({{source}}, Unset) else (None, str({{source}}).encode(), "text/plain")
{% else %}
{{ destination }} =  (None, str({{ source }}).encode(), "text/plain")
{% endif %}
{% endmacro %}